package de.hsel.itech.db;

import de.hsel.itech.db.pojo.*;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class OrderDB {

    private Database database;

    OrderDB(@NotNull Database database) {
        this.database = database;
    }

    @Nullable
    public Order get(long id) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tableOrder, id);
        assert entry != null;

        try {

            ResultSet rs = entry.getKey();
            List<OrderItem> items = getItems(id);
            Payment payment = database.payment().get(rs.getLong("payment"));
            Address address = database.address().get(rs.getLong("address"));
            Order order = new Order(id, database.user().get(rs.getLong("user")), items, rs.getDate("date"), rs.getLong("price"), payment, address);

            entry.getValue().close();
            return order;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    private List<OrderItem> getItems(long id) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetsById(database.tableOrderBook, "order", id);
        assert entry != null;

        List<OrderItem> items = new ArrayList<>();

        try {
            ResultSet rs = entry.getKey();
            while(rs.next()) {
                items.add(new OrderItem(database.book().get(rs.getLong("book")), rs.getInt("count")));
            }

            entry.getValue().close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return items;
    }

    public int insert(@NotNull Order order) {
        Connection connection = database.getConnection();
        assert connection != null;

        int insertCount = 0;

        try {
            PreparedStatement statement = connection.prepareStatement("INSERT INTO `order` (user, price, payment, date, address) VALUES (?, ?, ?, ?, ?)", Statement.RETURN_GENERATED_KEYS);
            statement.setLong(1, order.getUser().getId());
            statement.setLong(2, order.getPrice());
            statement.setLong(3, order.getPayment().getId());
            statement.setDate(4, order.getDate());
            statement.setLong(5, order.getAddress().getId());

            insertCount += statement.executeUpdate();
            ResultSet idResultSet = statement.getGeneratedKeys();
            long lastId = -1;
            while (idResultSet.next()) {
                lastId = idResultSet.getLong("id");
            }

            for(OrderItem item : order.getItems()) {
                PreparedStatement itemStatement = connection.prepareStatement("INSERT INTO order_book (`order`, book, count) VALUES (?, ?, ?)");
                itemStatement.setLong(1, lastId);
                itemStatement.setLong(2, item.getBook().getId());
                itemStatement.setInt(3, item.getCount());

                insertCount += itemStatement.executeUpdate();
            }

            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return insertCount;
    }

    public int delete(@NotNull Order order) {
        int deleteCount = 0;

        deleteCount += database.delete(order.getId(), database.tableOrder);
        deleteCount += database.delete(order.getId(), database.tableOrderBook, "order");

        return deleteCount;
    }
}